﻿/**
 * Progression 4
 * 
 * @author Copyright (C) 2007-2010 taka:nium.jp, All Rights Reserved.
 * @version 4.0.2
 * @see http://progression.jp/
 * 
 * Progression Libraries is dual licensed under the "Progression Library License" and "GPL".
 * http://progression.jp/license
 */
package jp.progression.casts.buttons {
	import jp.nium.core.debug.Logger;
	import jp.nium.core.L10N.L10NNiumMsg;
	import jp.nium.utils.ObjectUtil;
	import jp.progression.core.display.ButtonBase;
	import jp.progression.core.PackageInfo;
	import jp.progression.events.ManagerEvent;
	import jp.progression.scenes.SceneId;
	
	/**
	 * <span lang="ja">RootButton クラスは、現在のシーン構造も最も上に位置するルートシーンに移動するボタン機能を提供するコンポーネントクラスです。
	 * このクラスを使用するためには、CommandExecutor が実装される環境設定（BasicAppConfig、WebConfig、等）を使用する必要があります。</span>
	 * <span lang="en"></span>
	 * 
	 * @example <listing version="3.0">
	 * // RootButton インスタンスを作成する
	 * var cast:RootButton = new RootButton();
	 * cast.graphics.beginFill( 0x000000 );
	 * cast.graphics.drawRect( 0, 0, 100, 100 );
	 * cast.graphics.endFill();
	 * 
	 * // 関連付けたいマネージャー識別子を設定する
	 * cast.managerId = "index";
	 * 
	 * // SerialList コマンドを実行する
	 * new SerialList( null,
	 * 	// 画面に表示する
	 * 	new AddChild( this, cast ),
	 * 	
	 * 	// 画面から消去する
	 * 	new RemoveChild( this, cast )
	 * ).execute();
	 * </listing>
	 */
	public class RootButton extends ButtonBase {
		
		/**
		 * <span lang="ja">関連付けたい Progression 識別子を示すストリングを取得または設定します。</span>
		 * <span lang="en"></span>
		 */
		public function get managerId():String { return _managerId; }
		public function set managerId( value:String ):void {
			// 書式が正しければ
			if ( SceneId.validateName( value ) ) {
				// 変更されているかどうかを取得する
				var changed:Boolean = ( _managerId != value );
				
				// 更新する
				_managerId = value;
				
				// sceneId を設定する
				if ( changed ) {
					super.sceneId = new SceneId( "/" + value );
				}
			}
			else if ( value && PackageInfo.hasDebugger ) {
				Logger.error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_005 ).toString( value ) );
			}
		}
		private var _managerId:String;
		
		/**
		 * @private
		 */
		override public function get sceneId():SceneId { return super.sceneId; }
		override public function set sceneId( value:SceneId ):void { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_001 ).toString( "sceneId" ) ); }
		
		/**
		 * @private
		 */
		override public function get href():String { return super.href; }
		override public function set href( value:String ):void { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_001 ).toString( "href" ) ); }
		
		
		
		
		
		/**
		 * <span lang="ja">新しい RootButton インスタンスを作成します。</span>
		 * <span lang="en">Creates a new RootButton object.</span>
		 * 
		 * @param initObject
		 * <span lang="ja">設定したいプロパティを含んだオブジェクトです。</span>
		 * <span lang="en"></span>
		 */
		public function RootButton( initObject:Object = null ) {
			// 親クラスを初期化する
			super( initObject );
			
			// イベントリスナーを登録する
			super.addEventListener( ManagerEvent.MANAGER_ACTIVATE, _managerActivate, false, 0, true );
		}
		
		
		
		
		
		/**
		 * <span lang="ja">指定されたオブジェクトのストリング表現を返します。</span>
		 * <span lang="en">Returns the string representation of the specified object.</span>
		 * 
		 * @return
		 * <span lang="ja">オブジェクトのストリング表現です。</span>
		 * <span lang="en">A string representation of the object.</span>
		 */
		override public function toString():String {
			return ObjectUtil.formatToString( this, super.className, super.id ? "id" : null, "managerId" );
		}
		
		
		
		
		
		/**
		 * キャストオブジェクトと Progression インスタンスの関連付けがアクティブになったときに送出されます。
		 */
		private function _managerActivate( e:ManagerEvent ):void {
			// Progression 識別子を設定する
			managerId ||= super.manager.id;
		}
	}
}
